class MET(object):
    """
    The Metabolic Equivalent of Task (MET), or metabolic equivalent, is a measure expressing the energy cost of physical activities. METs are defined as the ratio of metabolic rate (and therefore the rate of energy consumption) during a specific physical activity to a reference metabolic rate.

    One MET is defined as 1 kcal/kg/hour and is roughly equivalent to the energy cost of sitting quietly. A MET also is defined as oxygen uptake in ml/kg/min with one MET equal to the oxygen cost of sitting quietly, equivalent to 3.5 ml/kg/min.

    1 MET = 1kcal/kg*h = 4.184 kJ/kg*h = 1.162 W/kg
    """
    __slots__ = ('value', 'code', 'description')

    def __init__(self, value, code, description):
        """
        args:
            value (float):
            code (str): The unique identifier for the activity
            description (str): Description of the activity
        """
        self.value = value
        self.code = code
        self.description = description

    def __float__(self):
        return self.value

    def __int__(self):
        return int(self.value)

    def __str__(self):
        return self.code

    def __repr__(self):
        return '<MET %s code=%s, description=%r value=%r>' % (id(self), self.code, self.description, self.value)

    def to_kcal(self, weight):
        """
        Estimates energy expenditure in kilocalories/minute

        Ehrman, Jonathan K. ACSM's Resource Manual for Guidelines for Exercise Testing and Prescription. 6th ed. Philadelphia: Wolters Kluwer Health/Lippincott Williams & Wilkins, 2010. Print.

        args:
            weight (float): body weight, given in kilograms

        Returns:
            float: kilocalories/minute
        """
        return (self.value * 3.5 * weight) / 200

    def karvonen(self, intensity):
        return intensity * (self.value - 1) + 1


MET.compendium = [
    MET(14.0, "01003", "bicycling, mountain, uphill, vigorous"),
    MET(16.0, "01004", "bicycling, mountain, competitive, racing"),
    MET(8.5, "01008", "bicycling, BMX"),
    MET(8.5, "01009", "bicycling, mountain, general"),
    MET(4.0, "01010", "bicycling, <10 mph, leisure, to work or for pleasure (Taylor Code 115)"),
    MET(6.8, "01011", "bicycling, to/from work, self selected pace"),
    MET(5.8, "01013", "bicycling, on dirt or farm road, moderate pace"),
    MET(7.5, "01015", "bicycling, general"),
    MET(3.5, "01018", "bicycling, leisure, 5.5 mph"),
    MET(6.8, "01020", "bicycling, 10-11.9 mph, leisure, slow, light effort"),
    MET(8.0, "01030", "bicycling, 12-13.9 mph, leisure, moderate effort"),
    MET(10.0, "01040", "bicycling, 14-15.9 mph, racing or leisure, fast, vigorous effort"),
    MET(12.0, "01050", "bicycling, 16-19 mph, racing/not drafting or > 19 mph drafting, very fast, racing general"),
    MET(15.8, "01060", "bicycling, > 20 mph, racing, not drafting"),
    MET(8.5, "01065", "bicycling, 12 mph, seated, hands on brake hoods or bar drops, 80 rpm"),
    MET(9.0, "01066", "bicycling, 12 mph, standing, hands on brake hoods, 60 rpm"),
    MET(5.0, "01070", "unicycling"),
    MET(2.3, "02001", "activity promoting video game (e.g., Wii Fit), light effort (e.g., balance, yoga)"),
    MET(3.8, "02003", "activity promoting video game (e.g., Wii Fit), moderate effort (e.g., aerobic, resistance)"),
    MET(7.2, "02005", "activity promoting video/arcade game (e.g., Exergaming, Dance Dance Revolution), vigorous effort"),
    MET(5.0, "02008", "army type obstacle course exercise, boot camp training program\\u00a0"),
    MET(7.0, "02010", "bicycling, stationary, general"),
    MET(3.5, "02011", "bicycling, stationary, 30-50 watts, very light to light effort"),
    MET(6.8, "02012", "bicycling, stationary, 90-100 watts, moderate to vigorous effort"),
    MET(8.8, "02013", "bicycling, stationary, 101-160 watts, vigorous effort"),
    MET(11.0, "02014", "bicycling, stationary, 161-200 watts, vigorous effort"),
    MET(14.0, "02015", "bicycling, stationary, 201-270 watts, very vigorous effort"),
    MET(4.8, "02017", "bicycling, stationary, 51-89 watts, light-to-moderate effort"),
    MET(8.5, "02019", "bicycling, stationary, RPM/Spin bike class"),
    MET(8.0, "02020", "calisthenics (e.g., push ups, sit ups, pull-ups, jumping jacks), vigorous effort"),
    MET(3.8, "02022", "calisthenics (e.g., push ups, sit ups, pull-ups, lunges), moderate effort"),
    MET(2.8, "02024", "calisthenics (e.g., situps, abdominal crunches), light effort"),
    MET(3.5, "02030", "calisthenics, light or moderate effort, general (example: back exercises), going up & down from floor (Taylor Code 150)"),
    MET(4.3, "02035", "circuit training, moderate effort"),
    MET(8.0, "02040", "circuit training, including kettlebells, some aerobic movement with minimal rest, general, vigorous intensity"),
    MET(3.5, "02045", "Curves exercise routines in women"),
    MET(5.0, "02048", "Elliptical trainer, moderate effort\\u00a0"),
    MET(6.0, "02050", "resistance training (weight lifting - free weight, nautilus or universal-type), power lifting or body building, vigorous effort (Taylor Code 210)"),
    MET(5.0, "02052", "resistance (weight) training, squats , slow or explosive effort"),
    MET(3.5, "02054", "resistance (weight) training, multiple exercises, 8-15 repetitions at varied resistance\\u00a0"),
    MET(5.5, "02060", "health club exercise, general (Taylor Code 160)"),
    MET(9.0, "02065", "stair-treadmill ergometer, general"),
    MET(11.0, "02068", "rope skipping, general"),
    MET(6.0, "02070", "rowing, stationary ergometer, general, vigorous effort"),
    MET(4.8, "02071", "rowing, stationary, general, moderate effort"),
    MET(7.0, "02072", "rowing, stationary, 100 watts, moderate effort"),
    MET(8.5, "02073", "rowing, stationary, 150 watts, vigorous effort"),
    MET(12.0, "02074", "rowing, stationary, 200 watts, very vigorous effort"),
    MET(6.8, "02080", "ski machine, general"),
    MET(11.0, "02085", "slide board exercise, general"),
    MET(6.0, "02090", "slimnastics, jazzercise"),
    MET(2.3, "02101", "stretching, mild"),
    MET(3.0, "02105", "pilates, general"),
    MET(6.8, "02110", "teaching exercise class (e.g., aerobic, water)"),
    MET(2.8, "02112", "therapeutic exercise ball, Fitball exercise"),
    MET(2.8, "02115", "upper body exercise, arm ergometer"),
    MET(4.3, "02117", "upper body exercise, stationary bicycle - Airdyne (arms only) 40 rpm, moderate"),
    MET(5.3, "02120", "water aerobics, water calisthenics, water exercise"),
    MET(1.3, "02135", "whirlpool, sitting"),
    MET(2.3, "02140", "video exercise workouts, TV conditioning programs (e.g., yoga, stretching), light effort"),
    MET(4.0, "02143", "video exercise workouts, TV conditioning programs (e.g., cardio-resistance), moderate effort"),
    MET(6.0, "02146", "video exercise workouts, TV conditioning programs (e.g., cardio-resistance), vigorous effort"),
    MET(2.5, "02150", "yoga, Hatha"),
    MET(4.0, "02160", "yoga, Power"),
    MET(2.0, "02170", "yoga, Nadisodhana"),
    MET(3.3, "02180", "yoga, Surya Namaskar"),
    MET(5.3, "02200", "native New Zealander physical activities (e.g., Haka Powhiri, Moteatea, Waita Tira, Whakawatea, etc.) , general, moderate effort"),
    MET(6.8, "02205", "native New Zealander physical activities (e.g., Haka, Taiahab), general, vigorous effort"),
    MET(5.0, "03010", "ballet, modern, or jazz, general, rehearsal or class"),
    MET(6.8, "03012", "ballet, modern, or jazz, performance, vigorous effort"),
    MET(4.8, "03014", "tap"),
    MET(7.3, "03015", "aerobic, general"),
    MET(7.5, "03016", "aerobic, step, with 6 - 8 inch step"),
    MET(9.5, "03017", "aerobic, step, with 10 - 12 inch step"),
    MET(5.5, "03018", "aerobic, step, with 4-inch step"),
    MET(8.5, "03019", "bench step class, general"),
    MET(5.0, "03020", "aerobic, low impact"),
    MET(7.3, "03021", "aerobic, high impact"),
    MET(10.0, "03022", "aerobic dance wearing 10-15 lb weights"),
    MET(4.5, "03025", "ethnic or cultural dancing (e.g., Greek, Middle Eastern, hula, salsa, merengue, bamba y plena, flamenco, belly, and swing)"),
    MET(5.5, "03030", "ballroom, fast (Taylor Code 125)"),
    MET(7.8, "03031", "general dancing (e.g., disco, folk, Irish step dancing, line dancing, polka, contra, country)"),
    MET(11.3, "03038", "ballroom dancing, competitive, general"),
    MET(3.0, "03040", "ballroom, slow (e.g., waltz, foxtrot, slow dancing, samba, tango, 19th century dance, mambo, cha cha)"),
    MET(5.5, "03050", "Anishinaabe Jingle Dancing"),
    MET(3.5, "03060", "Caribbean dance (Abakua, Beguine, Bellair, Bongo, Brukin's, Caribbean Quadrills, Dinki Mini, Gere, Gumbay, Ibo, Jonkonnu, Kumina, Oreisha, Jambu)"),
    MET(3.5, "04001", "fishing, general"),
    MET(4.5, "04005", "fishing, crab fishing"),
    MET(4.0, "04007", "fishing, catching fish with hands"),
    MET(4.3, "04010", "fishing related, digging worms, with shovel"),
    MET(4.0, "04020", "fishing from river bank and walking"),
    MET(2.0, "04030", "fishing from boat or canoe, sitting"),
    MET(3.5, "04040", "fishing from river bank, standing (Taylor Code 660)"),
    MET(6.0, "04050", "fishing in stream, in waders (Taylor Code 670)"),
    MET(2.0, "04060", "fishing, ice, sitting"),
    MET(1.8, "04061", "fishing, jog or line, standing, general"),
    MET(3.5, "04062", "fishing, dip net, setting net and retrieving fish, general"),
    MET(3.8, "04063", "fishing, set net, setting net and retrieving fish, general"),
    MET(3.0, "04064", "fishing, fishing wheel, setting net and retrieving fish, general"),
    MET(2.3, "04065", "fishing with a spear, standing"),
    MET(2.5, "04070", "hunting, bow and arrow, or crossbow"),
    MET(6.0, "04080", "hunting, deer, elk, large game (Taylor Code 170)"),
    MET(11.3, "04081", "hunting large game, dragging carcass"),
    MET(4.0, "04083", "hunting large marine animals"),
    MET(2.5, "04085", "hunting large game, from a hunting stand, limited walking"),
    MET(2.0, "04086", "hunting large game from a car, plane, or boat"),
    MET(2.5, "04090", "hunting, duck, wading"),
    MET(3.0, "04095", "hunting, flying fox, squirrel"),
    MET(5.0, "04100", "hunting, general"),
    MET(6.0, "04110", "hunting, pheasants or grouse (Taylor Code 680)"),
    MET(3.3, "04115", "hunting, birds"),
    MET(5.0, "04120", "hunting, rabbit, squirrel, prairie chick, raccoon, small game (Taylor Code 690)"),
    MET(3.3, "04123", "hunting, pigs, wild"),
    MET(2.0, "04124", "trapping game, general"),
    MET(9.5, "04125", "hunting, hiking with hunting gear"),
    MET(2.5, "04130", "pistol shooting or trap shooting, standing"),
    MET(2.3, "04140", "rifle exercises, shooting, lying down"),
    MET(2.5, "04145", "rifle exercises, shooting, kneeling or standing"),
    MET(3.3, "05010", "cleaning, sweeping carpet or floors, general"),
    MET(2.3, "05011", "cleaning, sweeping, slow, light effort"),
    MET(3.8, "05012", "cleaning, sweeping, slow, moderate effort"),
    MET(3.5, "05020", "cleaning, heavy or major (e.g. wash car, wash windows, clean garage), moderate effort"),
    MET(3.5, "05021", "cleaning, mopping, standing, moderate effort"),
    MET(3.2, "05022", "cleaning windows, washing windows, general"),
    MET(2.5, "05023", "mopping, standing, light effort"),
    MET(4.5, "05024", "polishing floors, standing, walking slowly, using electric polishing machine"),
    MET(2.8, "05025", "multiple household tasks all at once, light effort"),
    MET(3.5, "05026", "multiple household tasks all at once, moderate effort"),
    MET(4.3, "05027", "multiple household tasks all at once, vigorous effort"),
    MET(3.3, "05030", "cleaning, house or cabin, general, moderate effort"),
    MET(2.3, "05032", "dusting or polishing furniture, general"),
    MET(3.3, "05035", "kitchen activity, general, (e.g., cooking, washing dishes, cleaning up), moderate effort"),
    MET(2.5, "05040", "cleaning, general (straightening up, changing linen, carrying out trash, light effort"),
    MET(1.8, "05041", "wash dishes, standing or in general (not broken into stand/walk components)"),
    MET(2.5, "05042", "wash dishes, clearing dishes from table, walking, light effort"),
    MET(3.3, "05043", "vacuuming, general, moderate effort"),
    MET(3.0, "05044", "butchering animals, small"),
    MET(6.0, "05045", "butchering animal, large, vigorous effort"),
    MET(2.3, "05046", "cutting and smoking fish, drying fish or meat"),
    MET(4.0, "05048", "tanning hides, general"),
    MET(3.5, "05049", "cooking or food preparation, moderate effort"),
    MET(2.0, "05050", "cooking or food preparation - standing or sitting or in general (not broken into stand/walk components), manual appliances, light effort"),
    MET(2.5, "05051", "serving food, setting table, implied walking or standing"),
    MET(2.5, "05052", "cooking or food preparation, walking"),
    MET(2.5, "05053", "feeding household animals"),
    MET(2.5, "05055", "putting away groceries (e.g. carrying groceries, shopping without a grocery cart), carrying packages"),
    MET(7.5, "05056", "carrying groceries upstairs"),
    MET(3.0, "05057", "cooking Indian bread on an outside stove"),
    MET(2.3, "05060", "food shopping with or without a grocery cart, standing or walking"),
    MET(2.3, "05065", "non-food shopping, with or without a cart, standing or walking"),
    MET(1.8, "05070", "ironing"),
    MET(1.3, "05080", "knitting, sewing, light effort, wrapping presents, sitting"),
    MET(2.8, "05082", "sewing with a machine"),
    MET(2.0, "05090", "laundry, fold or hang clothes, put clothes in washer or dryer, packing suitcase, washing clothes by hand,implied standing, light effort"),
    MET(4.0, "05092", "laundry, hanging wash, washing clothes by hand, moderate effort"),
    MET(2.3, "05095", "laundry, putting away clothes, gathering clothes to pack, putting away laundry,implied walking"),
    MET(3.3, "05100", "making bed, changing linens"),
    MET(5.0, "05110", "maple syruping/sugar bushing (including carrying buckets, carrying wood)"),
    MET(5.8, "05120", "moving furniture, household items, carrying boxes"),
    MET(5.0, "05121", "moving, lifting light loads"),
    MET(4.8, "05125", "organizing room"),
    MET(3.5, "05130", "scrubbing floors, on hands and knees, scrubbing bathroom, bathtub, moderate effort"),
    MET(2.0, "05131", "scrubbing floors, on hands and knees, scrubbing bathroom, bathtub, light effort"),
    MET(6.5, "05132", "scrubbing floors, on hands and knees, scrubbing bathroom, bathtub, vigorous effort"),
    MET(4.0, "05140", "sweeping garage, sidewalk or outside of house"),
    MET(3.5, "05146", "standing, packing/unpacking boxes, occasional lifting of lightweight household items, loading or unloading items in car, moderate effort"),
    MET(3.0, "05147", "implied walking, putting away household items, moderate effort"),
    MET(2.5, "05148", "watering  plants"),
    MET(2.5, "05149", "building a fire inside"),
    MET(9.0, "05150", "moving household items upstairs, carrying boxes or furniture"),
    MET(2.0, "05160", "standing, light effort tasks (pump gas, change light bulb, etc.)"),
    MET(3.5, "05165", "walking, moderate effort tasks, non-cleaning (readying to leave, shut/lock doors, close windows, etc.)"),
    MET(2.2, "05170", "sitting, playing with child(ren), light effort, only active periods"),
    MET(2.8, "05171", "standing, playing with child(ren) light effort, only active periods"),
    MET(3.5, "05175", "walking/running, playing with child(ren), moderate effort, only active periods"),
    MET(5.8, "05180", "walking/running, playing with child(ren), vigorous effort, only active periods"),
    MET(3.0, "05181", "walking and carrying small child, child weighing 15 lbs or more"),
    MET(2.3, "05182", "walking, moderate effort tasks, non-cleaning (readying to leave, shut/lock doors, close windows, etc.)"),
    MET(2.0, "05183", "standing, holding child"),
    MET(2.5, "05184", "child care, infant, general"),
    MET(2.0, "05185", "child care, sitting/kneeling (e.g., dressing, bathing, grooming, feeding, occasional lifting of child), light effort, general"),
    MET(3.0, "05186", "child care, standing (e.g., dressing, bathing, grooming, feeding, occasional lifting of child), moderate effort"),
    MET(1.5, "05188", "reclining with baby"),
    MET(2.0, "05189", "breastfeeding, sitting or reclining"),
    MET(2.5, "05190", "sit, playing with animals, light effort, only active periods"),
    MET(2.8, "05191", "stand, playing with animals, light effort, only active periods"),
    MET(3.0, "05192", "walk/run, playing with animals, general, light effort, only active periods"),
    MET(4.0, "05193", "walk/run, playing with animals, moderate effort, only active periods"),
    MET(5.0, "05194", "walk/run, playing with animals, vigorous effort, only active periods"),
    MET(3.5, "05195", "standing, bathing dog"),
    MET(2.3, "05197", "animal care, household animals, general"),
    MET(4.0, "05200", "elder care, disabled adult, bathing, dressing, moving into and out of bed, only active periods "),
    MET(2.3, "05205", "elder care, disabled adult, feeding, combing hair, light effort, only active periods"),
    MET(3.0, "06010", "airplane repair"),
    MET(4.0, "06020", "automobile body work"),
    MET(3.3, "06030", "automobile repair, light or moderate effort"),
    MET(3.0, "06040", "carpentry, general, workshop (Taylor Code 620)"),
    MET(6.0, "06050", "carpentry, outside house, installing rain gutters (Taylor Code 640),carpentry, outside house, building a fence"),
    MET(3.8, "06052", "carpentry, outside house, building a fence"),
    MET(3.3, "06060", "carpentry, finishing or refinishing cabinets or furniture"),
    MET(6.0, "06070", "carpentry, sawing hardwood"),
    MET(4.0, "06072", "carpentry, home remodeling tasks, moderate effort"),
    MET(2.3, "06074", "carpentry, home remodeling tasks, light effort\\u00a0"),
    MET(5.0, "06080", "caulking, chinking log cabin"),
    MET(4.5, "06090", "caulking, except log cabin"),
    MET(5.0, "06100", "cleaning gutters"),
    MET(5.0, "06110", "excavating garage"),
    MET(5.0, "06120", "hanging storm windows"),
    MET(5.0, "06122", "hanging sheet rock inside house"),
    MET(3.0, "06124", "hammering nails"),
    MET(2.5, "06126", "home repair, general, light effort"),
    MET(4.5, "06127", "home repair, general, moderate effort"),
    MET(6.0, "06128", "home repair, general, vigorous effort"),
    MET(4.5, "06130", "laying or removing carpet"),
    MET(3.8, "06140", "laying tile or linoleum,repairing appliances"),
    MET(3.0, "06144", "repairing appliances"),
    MET(5.0, "06150", "painting, outside home (Taylor Code 650)"),
    MET(3.3, "06160", "painting inside house,wallpapering, scraping paint"),
    MET(4.5, "06165", "painting, (Taylor Code 630)"),
    MET(3.0, "06167", "plumbing, general"),
    MET(3.0, "06170", "put on and removal of tarp - sailboat"),
    MET(6.0, "06180", "roofing"),
    MET(4.5, "06190", "sanding floors with a power sander"),
    MET(4.5, "06200", "scraping and painting sailboat or powerboat"),
    MET(2.0, "06205", "sharpening tools"),
    MET(5.0, "06210", "spreading dirt with a shovel"),
    MET(4.5, "06220", "washing and waxing hull of sailboat or airplane"),
    MET(2.0, "06225", "washing and waxing car"),
    MET(4.5, "06230", "washing fence, painting fence, moderate effort"),
    MET(3.3, "06240", "wiring, tapping-splicing"),
    MET(1.0, "07010", "lying quietly and watching television"),
    MET(1.3, "07011", "lying quietly, doing nothing, lying in bed awake, listening to music (not talking or reading)"),
    MET(1.3, "07020", "sitting quietly and watching television"),
    MET(1.3, "07021", "sitting quietly, general"),
    MET(1.5, "07022", "sitting quietly, fidgeting, general, fidgeting hands"),
    MET(1.8, "07023", "sitting, fidgeting feet"),
    MET(1.3, "07024", "sitting, smoking"),
    MET(1.5, "07025", "sitting, listening to music (not talking or reading) or watching a movie in a theater"),
    MET(1.3, "07026", "sitting at a desk, resting head in hands"),
    MET(1.3, "07040", "standing quietly, standing in a line"),
    MET(1.8, "07041", "standing, fidgeting"),
    MET(1.3, "07050", "reclining, writing"),
    MET(1.3, "07060", "reclining, talking or talking on phone"),
    MET(1.3, "07070", "reclining, reading"),
    MET(1.0, "07075", "meditating"),
    MET(3.3, "08009", "carrying, loading or stacking wood, loading/unloading or carrying lumber, light-to-moderate effort"),
    MET(5.5, "08010", "carrying, loading or stacking wood, loading/unloading or carrying lumber"),
    MET(4.5, "08019", "chopping wood, splitting logs, moderate effort"),
    MET(6.3, "08020", "chopping wood, splitting logs, vigorous effort"),
    MET(3.5, "08025", "clearing light brush, thinning garden, moderate effort"),
    MET(6.3, "08030", "clearing brush/land, undergrowth, or ground, hauling branches, wheelbarrow chores, vigorous effort"),
    MET(5.0, "08040", "digging sandbox, shoveling sand"),
    MET(3.5, "08045", "digging, spading, filling garden, composting, light-to-moderate effort"),
    MET(5.0, "08050", "digging, spading, filling garden, compositing, (Taylor Code 590)"),
    MET(7.8, "08052", "digging, spading, filling garden, composting, vigorous effort"),
    MET(2.8, "08055", "driving tractor"),
    MET(8.3, "08057", "felling trees, large size"),
    MET(5.3, "08058", "felling trees, small-medium size"),
    MET(5.8, "08060", "gardening with heavy power tools, tilling a garden, chain saw"),
    MET(2.3, "08065", "gardening, using containers, older adults > 60 years"),
    MET(4.0, "08070", "irrigation channels, opening and closing ports"),
    MET(6.3, "08080", "laying crushed rock"),
    MET(5.0, "08090", "laying sod"),
    MET(5.5, "08095", "mowing lawn, general"),
    MET(2.5, "08100", "mowing lawn, riding mower (Taylor Code 550)"),
    MET(6.0, "08110", "mowing lawn, walk, hand mower (Taylor Code 570)"),
    MET(5.0, "08120", "mowing lawn, walk, power mower, moderate or vigorous effort"),
    MET(4.5, "08125", "mowing lawn, power mower, light or moderate effort (Taylor Code 590)"),
    MET(2.5, "08130", "operating snow blower, walking"),
    MET(2.0, "08135", "planting, potting, transplanting seedlings or plants, light effort"),
    MET(4.3, "08140", "planting seedlings, shrub, stooping, moderate effort"),
    MET(4.3, "08145", "planting crops or garden, stooping, moderate effort"),
    MET(4.5, "08150", "planting trees"),
    MET(3.8, "08160", "raking lawn or leaves, moderate effort"),
    MET(4.0, "08165", "raking lawn (Taylor Code 600)"),
    MET(4.0, "08170", "raking roof with snow rake"),
    MET(3.0, "08180", "riding snow blower"),
    MET(4.0, "08190", "sacking grass, leaves"),
    MET(5.5, "08192", "shoveling dirt or mud"),
    MET(5.3, "08195", "shoveling snow, by hand, moderate effort"),
    MET(6.0, "08200", "shovelling snow, by hand (Taylor Code 610)"),
    MET(7.5, "08202", "shoveling snow, by hand, vigorous effort"),
    MET(4.0, "08210", "trimming shrubs or trees, manual cutter"),
    MET(3.5, "08215", "trimming shrubs or trees, power cutter, using leaf blower, edge, moderate effort"),
    MET(3.0, "08220", "walking, applying fertilizer or seeding a lawn, push applicator"),
    MET(1.5, "08230", "watering lawn or garden, standing or walking"),
    MET(3.5, "08239", "weeding, cultivating garden, light-to-moderate effort"),
    MET(4.5, "08240", "weeding, cultivating garden (Taylor Code 580)"),
    MET(5.0, "08241", "weeding, cultivating garden, using a hoe, moderate-to-vigorous effort"),
    MET(3.8, "08245", "gardening, general, moderate effort"),
    MET(3.5, "08246", "picking fruit off trees, picking fruits/vegetables, moderate effort"),
    MET(4.5, "08248", "picking fruit off trees, gleaning fruits, picking fruits/vegetables, climbing ladder to pick fruit, vigorous effort"),
    MET(3.3, "08250", "implied walking/standing - picking up yard, light, picking flowers or vegetables"),
    MET(3.0, "08251", "walking, gathering gardening tools"),
    MET(5.5, "08255", "wheelbarrow, pushing garden cart or wheelbarrow"),
    MET(3.0, "08260", "yard work, general, light effort"),
    MET(4.0, "08261", "yard work, general, moderate effort"),
    MET(6.0, "08262", "yard work, general, vigorous effort"),
    MET(1.5, "09000", "board game playing, sitting"),
    MET(2.5, "09005", "casino gambling, standing"),
    MET(1.5, "09010", "card playing, sitting"),
    MET(1.5, "09013", "chess game, sitting"),
    MET(1.5, "09015", "copying documents, standing"),
    MET(1.8, "09020", "drawing, writing, painting, standing"),
    MET(1.0, "09025", "laughing, sitting"),
    MET(1.3, "09030", "sitting, reading, book, newspaper, etc."),
    MET(1.3, "09040", "sitting, writing, desk work, typing"),
    MET(1.0, "09045", "sitting, playing traditional video game, computer game"),
    MET(1.8, "09050", "standing, talking in person, on the phone, computer, or text messaging, light effort"),
    MET(1.5, "09055", "sitting, talking in person, on the phone, computer, or text messaging, light effort"),
    MET(1.3, "09060", "sitting, studying, general, including reading and/or writing, light effort"),
    MET(1.8, "09065", "sitting, in class, general, including note-taking or class discussion"),
    MET(1.8, "09070", "standing, reading"),
    MET(2.5, "09071", "standing, miscellaneous"),
    MET(1.8, "09075", "sitting, arts and crafts,\\u00a0 carving wood, weaving, spinning wool, light effort"),
    MET(3.0, "09080", "sitting, arts and crafts,\\u00a0 carving wood, weaving, spinning wool, moderate effort"),
    MET(2.5, "09085", "standing, arts and crafts, sand painting, carving, weaving, light effort"),
    MET(3.3, "09090", "standing, arts and crafts, sand painting, carving, weaving, moderate effort"),
    MET(3.5, "09095", "standing, arts and crafts, sand painting, carving, weaving, vigorous effort"),
    MET(1.8, "09100", "retreat/family reunion activities involving sitting, relaxing, talking, eating"),
    MET(3.0, "09101", "retreat/family reunion activities involving playing games with children"),
    MET(2.0, "09105", "touring/traveling/vacation involving riding in a vehicle"),
    MET(3.5, "09106", "touring/traveling/vacation involving walking"),
    MET(2.5, "09110", "camping involving standing, walking, sitting, light-to-moderate effort"),
    MET(1.5, "09115", "sitting at a sporting event, spectator"),
    MET(1.8, "10010", "accordion, sitting"),
    MET(2.3, "10020", "cello, sitting"),
    MET(2.3, "10030", "conducting orchestra, standing"),
    MET(3.8, "10040", "drums, sitting"),
    MET(3.0, "10045", "drumming (e.g., bongo, conga, benbe), moderate, sitting"),
    MET(2.0, "10050", "flute, sitting"),
    MET(1.8, "10060", "horn, standing"),
    MET(2.3, "10070", "piano, sitting"),
    MET(3.5, "10080", "trombone, standing"),
    MET(1.8, "10090", "trumpet, standing"),
    MET(2.5, "10100", "violin, sitting"),
    MET(1.8, "10110", "woodwind, sitting"),
    MET(2.0, "10120", "guitar, classical, folk, sitting"),
    MET(3.0, "10125", "guitar, rock and roll band, standing"),
    MET(4.0, "10130", "marching band, baton twirling, walking, moderate pace, general"),
    MET(5.5, "10131", "marching band, playing an instrument, walking, brisk pace, general"),
    MET(3.5, "10135", "marching band, drum major, walking"),
    MET(2.3, "11003", "active workstation, treadmill desk, walking"),
    MET(3.0, "11006", "airline flight attendant"),
    MET(4.0, "11010", "bakery, general, moderate effort"),
    MET(2.0, "11015", "bakery, light effort"),
    MET(2.3, "11020", "bookbinding"),
    MET(6.0, "11030", "building road, driving heavy machinery"),
    MET(2.0, "11035", "building road, directing traffic, standing"),
    MET(2.5, "11038", "carpentry, general, light effort"),
    MET(4.3, "11040", "carpentry, general, moderate effort"),
    MET(7.0, "11042", "carpentry, general, heavy or vigorous effort"),
    MET(8.0, "11050", "carrying heavy loads (e.g., bricks, tools)"),
    MET(8.0, "11060", "carrying moderate loads up stairs, moving boxes 25-49 lbs"),
    MET(4.0, "11070", "chambermaid, hotel housekeeper, making bed, cleaning bathroom, pushing cart"),
    MET(5.3, "11080", "coal mining, drilling coal, rock"),
    MET(5.0, "11090", "coal mining, erecting supports"),
    MET(5.5, "11100", "coal mining, general"),
    MET(6.3, "11110", "coal mining, shoveling coal"),
    MET(2.5, "11115", "cook, chef"),
    MET(4.0, "11120", "construction, outside, remodeling, new structures (e.g., roof repair, miscellaneous"),
    MET(2.3, "11125", "custodial work, light effort (e.g., cleaning sink and toilet, dusting, vacuuming, light cleaning)"),
    MET(3.8, "11126", "custodial work, moderate effort (e.g., electric buffer, feathering arena floors, mopping, taking out trash, vacuuming)"),
    MET(2.0, "11128", "driving delivery truck, taxi, shuttle bus, school bus"),
    MET(3.3, "11130", "electrical work (e.g., hook up wire, tapping-splicing)"),
    MET(1.8, "11135", "engineer (e.g., mechanical or electrical)"),
    MET(7.8, "11145", "farming, vigorous effort (e.g., baling hay, cleaning barn)"),
    MET(4.8, "11146", "farming, moderate effort (e.g., feeding animals, chasing cattle by walking and/or horseback, spreading manure, harvesting crops)"),
    MET(2.0, "11147", "farming, light effort (e.g., cleaning animal sheds, preparing animal feed)"),
    MET(2.8, "11170", "farming, driving tasks (e.g., driving tractor or harvester)"),
    MET(3.5, "11180", "farming, feeding small animals"),
    MET(4.3, "11190", "farming, feeding cattle, horses"),
    MET(4.3, "11191", "farming, hauling water for animals, general hauling water"),
    MET(4.5, "11192", "farming, taking care of animals (e.g., grooming, brushing, shearing sheep, assisting with birthing, medical care, branding), general"),
    MET(3.8, "11195", "farming, rice, planting, grain milling activities"),
    MET(3.5, "11210", "farming, milking by hand, cleaning pails, moderate effort"),
    MET(1.3, "11220", "farming, milking by machine, light effort"),
    MET(8.0, "11240", "fire fighter, general"),
    MET(6.8, "11244", "fire fighter, rescue victim, automobile accident, using pike pole"),
    MET(8.0, "11245", "fire fighter, raising and climbing ladder with full gear, simulated fire suppression"),
    MET(9.0, "11246", "fire fighter, hauling hoses on ground, carrying/hoisting equipment, breaking down walls, wearing full gear"),
    MET(3.5, "11247", "fishing, commercial, light effort"),
    MET(5.0, "11248", "fishing, commercial, moderate effort"),
    MET(7.0, "11249", "fishing, commercial, vigorous effort"),
    MET(17.5, "11250", "forestry, ax chopping, very fast, 1.25 kg axe, 51 blows/min, extremely vigorous effort"),
    MET(5.0, "11260", "forestry, ax chopping, slow, 1.25 kg axe, 19 blows/min, moderate effort"),
    MET(8.0, "11262", "forestry, ax chopping, fast, 1.25 kg axe, 35 blows/min, vigorous effort"),
    MET(4.5, "11264", "forestry, moderate effort (e.g., sawing wood with power saw, weeding, hoeing)"),
    MET(8.0, "11266", "forestry, vigorous effort (e.g., barking, felling, or trimming trees, carrying or stacking logs, felling trees, planting seeds, sawing lumber by hand )"),
    MET(4.5, "11370", "furriery"),
    MET(4.0, "11375", "garbage collector, walking, dumping bins into truck"),
    MET(1.8, "11378", "hairstylist (e.g., plaiting hair, manicure, make-up artist)"),
    MET(7.3, "11380", "horse grooming, including feeding, cleaning stalls, bathing, brushing, clipping, longeing and exercising horses."),
    MET(4.3, "11381", "horse, feeding, watering, cleaning stalls, implied walking and lifting loads"),
    MET(7.3, "11390", "horse racing, galloping"),
    MET(5.8, "11400", "horse racing, trotting"),
    MET(3.8, "11410", "horse racing, walking"),
    MET(3.0, "11413", "kitchen maid"),
    MET(4.0, "11415", "lawn keeper, yard work, general"),
    MET(3.3, "11418", "laundry worker"),
    MET(3.0, "11420", "locksmith"),
    MET(3.0, "11430", "machine tooling (e.g., machining, working sheet metal, machine fitter, operating lathe, welding) light-to-moderate effort"),
    MET(5.0, "11450", "machine tooling, operating punch press, moderate effort"),
    MET(1.8, "11472", "manager, property"),
    MET(2.8, "11475", "manual or unskilled labor, general, light effort"),
    MET(4.5, "11476", "manual or unskilled labor, general, moderate effort"),
    MET(6.5, "11477", "manual or unskilled labor, general, vigorous effort"),
    MET(4.3, "11480", "masonry, concrete, moderate effort"),
    MET(2.5, "11482", "masonry, concrete, light effort"),
    MET(4.0, "11485", "massage therapist, standing"),
    MET(7.5, "11490", "moving, carrying or pushing heavy objects, 75 lbs or more, only active time (e.g., desks, moving van work)"),
    MET(12.0, "11495", "skindiving or SCUBA diving as a frogman, Navy Seal"),
    MET(2.5, "11500", "operating heavy duty equipment, automated, not driving"),
    MET(4.5, "11510", "orange grove work, picking fruit"),
    MET(3.3, "11514", "painting,house, furniture, moderate effort"),
    MET(3.0, "11516", "plumbing activities"),
    MET(2.0, "11520", "printing, paper industry worker, standing"),
    MET(2.5, "11525", "police, directing traffic, standing"),
    MET(2.5, "11526", "police, driving a squad car, sitting"),
    MET(1.3, "11527", "police, riding in a squad car, sitting"),
    MET(4.0, "11528", "police, making an arrest, standing"),
    MET(2.3, "11529", "postal carrier, walking to deliver mail"),
    MET(2.0, "11530", "shoe repair, general"),
    MET(7.8, "11540", "shoveling, digging ditches"),
    MET(8.8, "11550", "shoveling, more than 16 pounds/minute, deep digging, vigorous effort"),
    MET(5.0, "11560", "shoveling, less than 10 pounds/minute, moderate effort"),
    MET(6.5, "11570", "shoveling, 10 to 15 pounds/minute, vigorous effort"),
    MET(1.5, "11580", "sitting tasks, light effort (e.g., office work, chemistry lab work, computer work, light assembly repair, watch repair, reading, desk work)"),
    MET(1.5, "11585", "sitting meetings, light effort, general, and/or with talking involved (e.g., eating at a business meeting)"),
    MET(2.5, "11590", "sitting tasks, moderate effort (e.g., pushing heavy levers, riding mower/forklift, crane operation)"),
    MET(2.8, "11593", "sitting, teaching stretching or yoga, or light effort exercise class"),
    MET(3.0, "11600", "standing tasks, light effort (e.g., bartending, store clerk, assembling, filing, duplicating, librarian, putting up a Christmas tree, standing and talking at work, changing clothes when teaching physical education,standing)"),
    MET(3.0, "11610", "standing, light/moderate effort (e.g., assemble/repair heavy parts, welding,stocking parts,auto repair,standing, packing boxes, nursing patient care)"),
    MET(4.5, "11615", "standing, moderate effort, lifting items continuously, 10 \\u2013 20 lbs, with limited walking or resting"),
    MET(3.5, "11620", "standing, moderate effort, intermittent lifting 50 lbs, hitch/twisting ropes"),
    MET(4.5, "11630", "standing, moderate/heavy tasks (e.g., lifting more than 50 lbs, masonry, painting, paper hanging)"),
    MET(5.3, "11708", "steel mill, moderate effort (e.g., fettling, forging, tipping molds)"),
    MET(8.3, "11710", "steel mill, vigorous effort (e.g., hand rolling, merchant mill rolling, removing slag, tending furnace)"),
    MET(2.3, "11720", "tailoring, cutting fabric"),
    MET(2.5, "11730", "tailoring, general"),
    MET(1.8, "11740", "tailoring, hand sewing"),
    MET(2.5, "11750", "tailoring, machine sewing"),
    MET(3.5, "11760", "tailoring, pressing"),
    MET(2.0, "11763", "tailoring, weaving, light effort (e.g., finishing operations, washing, dyeing, inspecting cloth, counting yards, paperwork)"),
    MET(4.0, "11765", "tailoring, weaving, moderate effort (e.g., spinning and weaving operations, delivering boxes of yam to spinners, loading of warp bean, pinwinding, conewinding, warping, cloth cutting)"),
    MET(6.5, "11766", "truck driving, loading and unloading truck, tying down load, standing, walking and carrying heavy loads"),
    MET(1.3, "11770", "typing, electric, manual or computer"),
    MET(6.3, "11780", "using heavy power tools such as pneumatic tools (e.g., jackhammers, drills)"),
    MET(8.0, "11790", "using heavy tools (not power) such as shovel, pick, tunnel bar, spade"),
    MET(2.0, "11791", "walking on job, less than 2.0 mph, very slow speed, in office or lab area"),
    MET(3.5, "11792", "walking on job, 3.0 mph, in office, moderate speed, not carrying anything"),
    MET(4.3, "11793", "walking on job, 3.5 mph, in office, brisk speed, not carrying anything"),
    MET(3.5, "11795", "walking on job, 2.5 mph, slow speed and carrying light objects less than 25 pounds"),
    MET(3.0, "11796", "walking, gathering things at work, ready to leave"),
    MET(3.8, "11797", "walking, 2.5 mph, slow speed, carrying heavy objects more than 25 lbs"),
    MET(4.5, "11800", "walking, 3.0 mph, moderately and carrying light objects less than 25 lbs"),
    MET(3.5, "11805", "walking, pushing a wheelchair"),
    MET(4.8, "11810", "walking, 3.5 mph, briskly and carrying objects less than 25 pounds"),
    MET(5.0, "11820", "walking or walk downstairs or standing, carrying objects about 25 to 49 pounds"),
    MET(6.5, "11830", "walking or walk downstairs or standing, carrying objects about 50 to 74 pounds"),
    MET(7.5, "11840", "walking or walk downstairs or standing, carrying objects about 75 to 99 pounds"),
    MET(8.5, "11850", "walking or walk downstairs or standing, carrying objects about 100 pounds or over"),
    MET(3.0, "11870", "working in scene shop, theater actor, backstage employee"),
    MET(6.0, "12010", "jog/walk combination (jogging component of less than 10 minutes) (Taylor Code 180)"),
    MET(7.0, "12020", "jogging, general"),
    MET(8.0, "12025", "jogging, in place"),
    MET(4.5, "12027", "jogging, on a mini-tramp"),
    MET(6.0, "12029", "running, 4 mph (15 min/mile)"),
    MET(8.3, "12030", "running, 5 mph (12 min/mile)"),
    MET(9.0, "12040", "running, 5.2 mph (11.5 min/mile)"),
    MET(9.8, "12050", "running, 6 mph (10 min/mile)"),
    MET(10.5, "12060", "running, 6.7 mph (9 min/mile)"),
    MET(11.0, "12070", "running, 7 mph (8.5 min/mile)"),
    MET(11.8, "12080", "running, 7.5 mph (8 min/mile)"),
    MET(11.8, "12090", "running, 8 mph (7.5 min/mile)"),
    MET(12.3, "12100", "running, 8.6 mph (7 min/mile)"),
    MET(12.8, "12110", "running, 9 mph (6.5 min/mile)"),
    MET(14.5, "12120", "running, 10 mph (6 min/mile)"),
    MET(16.0, "12130", "running, 11 mph (5.5 min/mile)"),
    MET(19.0, "12132", "running, 12 mph (5 min/mile)"),
    MET(19.8, "12134", "running, 13 mph (4.6 min/mile)"),
    MET(23.0, "12135", "running, 14 mph (4.3 min/mile)"),
    MET(9.0, "12140", "running, cross country"),
    MET(8.0, "12150", "running, (Taylor code 200)"),
    MET(15.0, "12170", "running, stairs, up"),
    MET(10.0, "12180", "running, on a track, team practice"),
    MET(8.0, "12190", "running, training, pushing a wheelchair or baby carrier"),
    MET(13.3, "12200", "running, marathon"),
    MET(2.3, "13000", "getting ready for bed, general, standing"),
    MET(1.8, "13009", "sitting on toilet, eliminating while standing or squating"),
    MET(1.5, "13010", "bathing, sitting"),
    MET(2.5, "13020", "dressing, undressing, standing or sitting"),
    MET(1.5, "13030", "eating, sitting"),
    MET(2.0, "13035", "talking and eating or eating only, standing"),
    MET(1.5, "13036", "taking medication, sitting or standing"),
    MET(2.0, "13040", "grooming, washing hands, shaving, brushing teeth, putting on make-up, sitting or standing"),
    MET(2.5, "13045", "hairstyling, standing"),
    MET(1.3, "13046", "having hair or nails done by someone else, sitting"),
    MET(2.0, "13050", "showering, toweling off, standing"),
    MET(2.8, "14010", "active, vigorous effort"),
    MET(1.8, "14020", "general, moderate effort"),
    MET(1.3, "14030", "passive, light effort, kissing, hugging"),
    MET(5.5, "15000", "Alaska Native Games, Eskimo Olympics, general"),
    MET(4.3, "15010", "archery, non-hunting"),
    MET(7.0, "15020", "badminton, competitive (Taylor Code 450)"),
    MET(5.5, "15030", "badminton, social singles and doubles, general"),
    MET(8.0, "15040", "basketball, game (Taylor Code 490)"),
    MET(6.0, "15050", "basketball, non-game, general (Taylor Code 480)"),
    MET(6.5, "15055", "basketball, general"),
    MET(7.0, "15060", "basketball, officiating (Taylor Code 500)"),
    MET(4.5, "15070", "basketball, shooting baskets"),
    MET(9.3, "15072", "basketball, drills, practice"),
    MET(7.8, "15075", "basketball, wheelchair"),
    MET(2.5, "15080", "billiards"),
    MET(3.0, "15090", "bowling (Taylor Code 390)"),
    MET(3.8, "15092", "bowling, indoor, bowling alley"),
    MET(12.8, "15100", "boxing, in ring, general"),
    MET(5.5, "15110", "boxing, punching bag"),
    MET(7.8, "15120", "boxing, sparring"),
    MET(7.0, "15130", "broomball"),
    MET(5.8, "15135", "children\\u2019s games, adults playing (e.g., hopscotch, 4-square, dodge ball, playground apparatus, t-ball, tetherball, marbles, jacks, arcade games), moderate effort"),
    MET(6.0, "15138", "cheerleading, gymnastic moves, competitive"),
    MET(4.0, "15140", "coaching, football, soccer, basketball, baseball, swimming, etc."),
    MET(8.0, "15142", "coaching, actively playing sport with players"),
    MET(4.8, "15150", "cricket, batting, bowling, fielding"),
    MET(3.3, "15160", "croquet"),
    MET(4.0, "15170", "curling"),
    MET(2.5, "15180", "darts, wall or lawn"),
    MET(6.0, "15190", "drag racing, pushing or driving a car"),
    MET(8.5, "15192", "auto racing, open wheel"),
    MET(6.0, "15200", "fencing"),
    MET(8.0, "15210", "football, competitive"),
    MET(8.0, "15230", "football, touch, flag, general (Taylor Code 510)"),
    MET(4.0, "15232", "football, touch, flag, light effort"),
    MET(2.5, "15235", "football or baseball, playing catch"),
    MET(3.0, "15240", "frisbee playing, general"),
    MET(8.0, "15250", "frisbee, ultimate"),
    MET(4.8, "15255", "golf, general"),
    MET(4.3, "15265", "golf, walking, carrying clubs"),
    MET(3.0, "15270", "golf, miniature, driving range"),
    MET(5.3, "15285", "golf, walking, pulling clubs"),
    MET(3.5, "15290", "golf, using power cart (Taylor Code 070)"),
    MET(3.8, "15300", "gymnastics, general"),
    MET(4.0, "15310", "hacky sack"),
    MET(12.0, "15320", "handball, general (Taylor Code 520)"),
    MET(8.0, "15330", "handball, team"),
    MET(4.0, "15335", "high ropes course, multiple elements"),
    MET(3.5, "15340", "hang gliding"),
    MET(7.8, "15350", "hockey, field"),
    MET(8.0, "15360", "hockey, ice, general"),
    MET(10.0, "15362", "hockey, ice, competitive"),
    MET(5.5, "15370", "horseback riding, general"),
    MET(4.3, "15375", "horse chores, feeding, watering, cleaning stalls, implied walking and lifting loads"),
    MET(4.5, "15380", "saddling, cleaning, grooming, harnessing and unharnessing horse"),
    MET(5.8, "15390", "horseback riding, trotting"),
    MET(7.3, "15395", "horseback riding, canter or gallop"),
    MET(3.8, "15400", "horseback riding,walking"),
    MET(9.0, "15402", "horseback riding, jumping"),
    MET(1.8, "15408", "horse cart, driving, standing or sitting"),
    MET(3.0, "15410", "horseshoe pitching, quoits"),
    MET(12.0, "15420", "jai alai"),
    MET(5.3, "15425", "martial arts, different types, slower pace, novice performers, practice"),
    MET(10.3, "15430", "martial arts, different types, moderate pace (e.g., judo, jujitsu, karate, kick boxing, tae kwan do, tai-bo, Muay Thai boxing)"),
    MET(4.0, "15440", "juggling"),
    MET(7.0, "15450", "kickball"),
    MET(8.0, "15460", "lacrosse"),
    MET(3.3, "15465", "lawn bowling, bocce ball, outdoor"),
    MET(4.0, "15470", "moto-cross, off-road motor sports, all-terrain vehicle, general"),
    MET(9.0, "15480", "orienteering"),
    MET(10.0, "15490", "paddleball, competitive"),
    MET(6.0, "15500", "paddleball, casual, general (Taylor Code 460)"),
    MET(8.0, "15510", "polo, on horseback"),
    MET(10.0, "15520", "racquetball, competitive"),
    MET(7.0, "15530", "racquetball, general (Taylor Code 470)"),
    MET(8.0, "15533", "rock or mountain climbing (Taylor Code 470) "),
    MET(7.5, "15535", "rock climbing, ascending rock, high difficulty"),
    MET(5.8, "15537", "rock climbing, ascending or traversing rock, low-to-moderate difficulty"),
    MET(5.0, "15540", "rock climbing, rappelling"),
    MET(4.0, "15542", "rodeo sports, general, light effort"),
    MET(5.5, "15544", "rodeo sports, general, moderate effort"),
    MET(7.0, "15546", "rodeo sports, general, vigorous effort"),
    MET(12.3, "15550", "rope jumping, fast pace, 120-160 skips/min"),
    MET(11.8, "15551", "rope jumping, moderate pace, 100-120 skips/min, general,\\u00a0 2 foot skip, plain bounce"),
    MET(8.8, "15552", "rope jumping, slow pace, < 100 skips/min, 2 foot skip, rhythm bounce"),
    MET(8.3, "15560", "rugby, union, team, competitive"),
    MET(6.3, "15562", "rugby, touch, non-competitive"),
    MET(3.0, "15570", "shuffleboard"),
    MET(5.0, "15580", "skateboarding, general, moderate effort"),
    MET(6.0, "15582", "skateboarding, competitive, vigorous effort"),
    MET(7.0, "15590", "skating, roller (Taylor Code 360)"),
    MET(7.5, "15591", "rollerblading, in-line skating, 14.4 km/h (9.0 mph), recreational pace"),
    MET(9.8, "15592", "rollerblading, in-line skating, 17.7 km/h (11.0 mph), moderate pace, exercise training"),
    MET(12.3, "15593", "rollerblading, in-line skating, 21.0 to 21.7 km/h (13.0 to 13.6 mph), fast pace, exercise training"),
    MET(14.0, "15594", "rollerblading, in-line skating, 24.0 km/h (15.0 mph), maximal effort"),
    MET(3.5, "15600", "skydiving, base jumping, bungee jumping"),
    MET(10.0, "15605", "soccer, competitive"),
    MET(7.0, "15610", "soccer, casual, general (Taylor Code 540)"),
    MET(5.0, "15620", "softball or baseball, fast or slow pitch, general (Taylor Code 440)"),
    MET(4.0, "15625", "softball, practice"),
    MET(4.0, "15630", "softball, officiating"),
    MET(6.0, "15640", "softball,pitching"),
    MET(3.3, "15645", "sports spectator, very excited, emotional, physically moving\\u00a0"),
    MET(12.0, "15650", "squash (Taylor Code 530)"),
    MET(7.3, "15652", "squash, general"),
    MET(4.0, "15660", "table tennis, ping pong (Taylor Code 410)"),
    MET(3.0, "15670", "tai chi, qi gong, general"),
    MET(1.5, "15672", "tai chi, qi gong, sitting, light effort"),
    MET(7.3, "15675", "tennis, general"),
    MET(6.0, "15680", "tennis, doubles (Taylor Code 430)"),
    MET(4.5, "15685", "tennis, doubles"),
    MET(8.0, "15690", "tennis, singles (Taylor Code 420)"),
    MET(5.0, "15695", "tennis, hitting balls, non-game play, moderate effort"),
    MET(3.5, "15700", "trampoline, recreational"),
    MET(4.5, "15702", "trampoline, competitive"),
    MET(4.0, "15710", "volleyball (Taylor Code 400)"),
    MET(6.0, "15711", "volleyball, competitive, in gymnasium"),
    MET(3.0, "15720", "volleyball, non-competitive, 6 - 9 member team, general"),
    MET(8.0, "15725", "volleyball, beach, in sand"),
    MET(6.0, "15730", "wrestling (one match = 5 minutes)"),
    MET(7.0, "15731", "wallyball, general"),
    MET(4.0, "15732", "track and field (e.g., shot, discus, hammer throw)"),
    MET(6.0, "15733", "track and field (e.g., high jump, long jump, triple jump, javelin, pole vault)"),
    MET(10.0, "15734", "track and field (e.g., steeplechase, hurdles)"),
    MET(2.5, "16010", "automobile or light truck (not a semi) driving"),
    MET(1.3, "16015", "riding in a car or truck"),
    MET(1.3, "16016", "riding in a bus or train"),
    MET(1.8, "16020", "flying airplane or helicopter"),
    MET(3.5, "16030", "motor scooter, motorcycle"),
    MET(6.3, "16035", "pulling rickshaw"),
    MET(6.0, "16040", "pushing plane in and out of hangar"),
    MET(2.5, "16050", "truck, semi, tractor, > 1 ton, or bus, driving"),
    MET(3.5, "16060", "walking for transportation, 2.8-3.2 mph, level, moderate pace, firm surface"),
    MET(7.0, "17010", "backpacking (Taylor Code 050)"),
    MET(7.8, "17012", "backpacking, hiking or organized walking with a daypack"),
    MET(5.0, "17020", "carrying 15 pound load (e.g. suitcase), level ground or downstairs"),
    MET(2.3, "17021", "carrying 15 lb child, slow walking"),
    MET(8.3, "17025", "carrying load upstairs, general"),
    MET(5.0, "17026", "carrying 1 to 15 lb load, upstairs"),
    MET(6.0, "17027", "carrying 16 to 24 lb load, upstairs"),
    MET(8.0, "17028", "carrying 25 to 49 lb load, upstairs"),
    MET(10.0, "17029", "carrying 50 to 74 lb load, upstairs"),
    MET(12.0, "17030", "carrying > 74 lb load, upstairs"),
    MET(3.5, "17031", "loading /unloading a car, implied walking"),
    MET(6.3, "17033", "climbing hills, no load"),
    MET(6.5, "17035", "climbing hills with 0 to 9 lb load"),
    MET(7.3, "17040", "climbing hills with 10 to 20 lb load"),
    MET(8.3, "17050", "climbing hills with 21 to 42 lb load"),
    MET(9.0, "17060", "climbing hills with 42+ lb load"),
    MET(3.5, "17070", "descending stairs"),
    MET(6.0, "17080", "hiking, cross country (Taylor Code 040)"),
    MET(5.3, "17082", "hiking or walking at a normal pace through fields and hillsides"),
    MET(2.5, "17085", "bird watching, slow walk"),
    MET(4.5, "17088", "marching, moderate speed, military, no pack"),
    MET(8.0, "17090", "marching rapidly, military, no pack"),
    MET(4.0, "17100", "pushing or pulling stroller with child or walking with children, 2.5 to 3.1 mph"),
    MET(3.8, "17105", "pushing a wheelchair, non-occupational\\u00a0"),
    MET(6.5, "17110", "race walking"),
    MET(8.0, "17130", "stair climbing, using or climbing up ladder (Taylor Code 030)"),
    MET(4.0, "17133", "stair climbing, slow pace"),
    MET(8.8, "17134", "stair climbing, fast pace"),
    MET(5.0, "17140", "using crutches"),
    MET(2.0, "17150", "walking, household"),
    MET(2.0, "17151", "walking, less than 2.0 mph, level, strolling, very slow"),
    MET(2.8, "17152", "walking, 2.0 mph, level, slow pace, firm surface"),
    MET(3.5, "17160", "walking for pleasure (Taylor Code 010)"),
    MET(2.5, "17161", "walking from house to car or bus, from car or bus to go places, from car or bus to and from the worksite"),
    MET(2.5, "17162", "walking to neighbor\\u2019s house or family\\u2019s house for social reasons"),
    MET(3.0, "17165", "walking the dog"),
    MET(3.0, "17170", "walking, 2.5 mph, level, firm surface"),
    MET(3.3, "17180", "walking, 2.5 mph, downhill"),
    MET(3.5, "17190", "walking, 2.8 to 3.2 mph, level, moderate pace, firm surface"),
    MET(4.3, "17200", "walking, 3.5 mph, level, brisk, firm surface, walking for exercise"),
    MET(5.3, "17210", "walking, 2.9 to 3.5 mph, uphill, 1 to 5% grade"),
    MET(8.0, "17211", "walking, 2.9 to 3.5 mph, uphill, 6% to 15% grade"),
    MET(5.0, "17220", "walking, 4.0 mph, level, firm surface, very brisk pace"),
    MET(7.0, "17230", "walking, 4.5 mph, level, firm surface, very, very brisk"),
    MET(8.3, "17231", "walking, 5.0 mph, level, firm surface"),
    MET(9.8, "17235", "walking, 5.0 mph, uphill, 3% grade"),
    MET(3.5, "17250", "walking, for pleasure, work break"),
    MET(4.8, "17260", "walking, grass track"),
    MET(4.5, "17262", "walking, normal pace, plowed field or sand"),
    MET(4.0, "17270", "walking, to work or class (Taylor Code 015)"),
    MET(2.5, "17280", "walking, to and from an outhouse"),
    MET(4.8, "17302", "walking, for exercise, 3.5 to 4 mph, with ski poles, Nordic walking, level, moderate pace"),
    MET(9.5, "17305", "walking, for exercise, 5.0 mph, with ski poles, Nordic walking, level, fast pace"),
    MET(6.8, "17310", "walking, for exercise, with ski poles, Nordic walking, uphill"),
    MET(6.0, "17320", "walking, backwards, 3.5 mph, level"),
    MET(8.0, "17325", "walking, backwards, 3.5 mph, uphill, 5% grade"),
    MET(2.5, "18010", "boating, power, driving"),
    MET(1.3, "18012", "boating, power, passenger, light"),
    MET(4.0, "18020", "canoeing, on camping trip (Taylor Code 270)"),
    MET(3.3, "18025", "canoeing, harvesting wild rice, knocking rice off the stalks"),
    MET(7.0, "18030", "canoeing, portaging"),
    MET(2.8, "18040", "canoeing, rowing, 2.0-3.9 mph, light effort"),
    MET(5.8, "18050", "canoeing, rowing, 4.0-5.9 mph, moderate effort"),
    MET(12.5, "18060", "canoeing, rowing, kayaking, competition, >6 mph, vigorous effort"),
    MET(3.5, "18070", "canoeing, rowing, for pleasure, general (Taylor Code 250)"),
    MET(12.0, "18080", "canoeing, rowing, in competition, or crew or sculling (Taylor Code 260)"),
    MET(3.0, "18090", "diving, springboard or platform"),
    MET(5.0, "18100", "kayaking, moderate effort"),
    MET(4.0, "18110", "paddle boat"),
    MET(3.0, "18120", "sailing, boat and board sailing, windsurfing, ice sailing, general (Taylor Code 235)"),
    MET(4.5, "18130", "sailing, in competition"),
    MET(3.3, "18140", "sailing, Sunfish/Laser/Hobby Cat, Keel boats, ocean sailing, yachting, leisure"),
    MET(6.0, "18150", "skiing, water or wakeboarding (Taylor Code 220)"),
    MET(7.0, "18160", "jet skiing, driving, in water"),
    MET(15.8, "18180", "skindiving, fast"),
    MET(11.8, "18190", "skindiving, moderate"),
    MET(7.0, "18200", "skindiving, scuba diving, general (Taylor Code 310)"),
    MET(5.0, "18210", "snorkeling (Taylor Code 310)"),
    MET(3.0, "18220", "surfing, body or board, general"),
    MET(5.0, "18222", "surfing, body or board, competitive"),
    MET(6.0, "18225", "paddle boarding, standing"),
    MET(9.8, "18230", "swimming laps, freestyle, fast, vigorous effort"),
    MET(5.8, "18240", "swimming laps, freestyle, front crawl, slow, light or moderate effort"),
    MET(9.5, "18250", "swimming, backstroke, general, training or competition"),
    MET(4.8, "18255", "swimming, backstroke, recreational"),
    MET(10.3, "18260", "swimming, breaststroke, general, training or competition"),
    MET(5.3, "18265", "swimming, breaststroke, recreational"),
    MET(13.8, "18270", "swimming, butterfly, general"),
    MET(10.0, "18280", "swimming, crawl, fast speed, ~75 yards/minute, vigorous effort"),
    MET(8.3, "18290", "swimming, crawl, medium speed, ~50 yards/minute, vigorous effort"),
    MET(6.0, "18300", "swimming, lake, ocean, river (Taylor Codes 280, 295)"),
    MET(6.0, "18310", "swimming, leisurely, not lap swimming, general"),
    MET(7.0, "18320", "swimming, sidestroke, general"),
    MET(8.0, "18330", "swimming, synchronized"),
    MET(9.8, "18340", "swimming, treading water, fast, vigorous effort"),
    MET(3.5, "18350", "swimming, treading water, moderate effort, general"),
    MET(2.3, "18352", "tubing, floating on a river, general"),
    MET(5.5, "18355", "water aerobics, water calisthenics"),
    MET(10.0, "18360", "water polo"),
    MET(3.0, "18365", "water volleyball"),
    MET(2.5, "18367", "water walking, light effort, slow pace"),
    MET(4.5, "18368", "water walking, moderate effort, moderate pace"),
    MET(6.8, "18369", "water walking, vigorous effort, brisk pace"),
    MET(5.0, "18370", "whitewater rafting, kayaking, or canoeing"),
    MET(5.0, "18380", "windsurfing, not pumping for speed"),
    MET(11.0, "18385", "windsurfing or kitesurfing, crossing trial"),
    MET(13.5, "18390", "windsurfing, competition, pumping for speed"),
    MET(7.5, "19005", "dog sledding, mushing"),
    MET(2.5, "19006", "dog sledding, passenger"),
    MET(6.0, "19010", "moving ice house, set up/drill holes"),
    MET(2.0, "19011", "ice fishing, sitting"),
    MET(14.0, "19018", "skating, ice dancing"),
    MET(5.5, "19020", "skating, ice, 9 mph or less"),
    MET(7.0, "19030", "skating, ice, general (Taylor Code 360)"),
    MET(9.0, "19040", "skating, ice, rapidly, more than 9 mph, not competitive"),
    MET(13.3, "19050", "skating, speed, competitive"),
    MET(7.0, "19060", "ski jumping, climb up carrying skis"),
    MET(7.0, "19075", "skiing, general"),
    MET(6.8, "19080", "skiing, cross country, 2.5 mph, slow or light effort, ski walking"),
    MET(9.0, "19090", "skiing, cross country, 4.0-4.9 mph, moderate speed and effort, general"),
    MET(12.5, "19100", "skiing, cross country, 5.0-7.9 mph, brisk speed, vigorous effort"),
    MET(15.0, "19110", "skiing, cross country, >8.0 mph, elite skier, racing"),
    MET(15.5, "19130", "skiing, cross country, hard snow, uphill, maximum, snow mountaineering"),
    MET(13.3, "19135", "skiing, cross-country, skating"),
    MET(13.5, "19140", "skiing, cross-country, biathlon, skating technique"),
    MET(4.3, "19150", "skiing, downhill, alpine or snowboarding, light effort, active time only"),
    MET(5.3, "19160", "skiing, downhill, alpine or snowboarding, moderate effort, general, active time only"),
    MET(8.0, "19170", "skiing, downhill, vigorous effort, racing"),
    MET(12.5, "19175", "skiing, roller, elite racers"),
    MET(7.0, "19180", "sledding, tobogganing, bobsledding, luge (Taylor Code 370)"),
    MET(5.3, "19190", "snow shoeing, moderate effort"),
    MET(10.0, "19192", "snow shoeing, vigorous effort"),
    MET(3.5, "19200", "snowmobiling, driving, moderate"),
    MET(2.0, "19202", "snowmobiling, passenger"),
    MET(5.3, "19252", "snow shoveling, by hand, moderate effort"),
    MET(7.5, "19254", "snow shoveling, by hand, vigorous effort"),
    MET(2.5, "19260", "snow blower, walking and pushing"),
    MET(1.3, "20000", "sitting in church, in service, attending a ceremony, sitting quietly"),
    MET(2.0, "20001", "sitting, playing an instrument at church"),
    MET(1.8, "20005", "sitting in church, talking or singing, attending a ceremony, sitting, active participation"),
    MET(1.3, "20010", "sitting, reading religious materials at home"),
    MET(1.3, "20015", "standing quietly in church, attending a ceremony"),
    MET(2.0, "20020", "standing, singing in church, attending a ceremony, standing, active participation"),
    MET(1.3, "20025", "kneeling in church or at home, praying"),
    MET(1.8, "20030", "standing, talking in church"),
    MET(2.0, "20035", "walking in church"),
    MET(2.0, "20036", "walking, less than 2.0 mph, very slow"),
    MET(3.5, "20037", "walking, 3.0 mph, moderate speed, not carrying anything"),
    MET(4.3, "20038", "walking, 3.5 mph, brisk speed, not carrying anything"),
    MET(2.0, "20039", "walk/stand combination for religious purposes, usher"),
    MET(5.0, "20040", "praise with dance or run, spiritual dancing in church"),
    MET(2.5, "20045", "serving food at church"),
    MET(2.0, "20046", "preparing food at church"),
    MET(3.3, "20047", "washing dishes, cleaning kitchen at church"),
    MET(1.5, "20050", "eating at church"),
    MET(2.0, "20055", "eating/talking at church or standing eating, American Indian Feast days"),
    MET(3.3, "20060", "cleaning church"),
    MET(4.0, "20061", "general yard work at church"),
    MET(3.5, "20065", "standing, moderate effort (e.g., lifting heavy objects, assembling at fast rate)"),
    MET(4.5, "20095", "standing, moderate-to-heavy effort, manual labor, lifting \\u2265 50 lbs, heavy maintenance"),
    MET(1.3, "20100", "typing, electric, manual, or computer"),
    MET(1.5, "21000", "sitting, meeting, general, and/or with talking involved"),
    MET(1.5, "21005", "sitting, light office work, in general"),
    MET(2.5, "21010", "sitting, moderate work"),
    MET(2.3, "21015", "standing, light work (filing, talking, assembling)"),
    MET(2.0, "21016", "sitting, child care, only active periods"),
    MET(3.0, "21017", "standing, child care, only active periods"),
    MET(3.5, "21018", "walk/run play with children, moderate, only active periods"),
    MET(5.8, "21019", "walk/run play with children, vigorous, only active periods"),
    MET(3.0, "21020", "standing, light/moderate work (e.g., pack boxes, assemble/repair, set up chairs/furniture)"),
    MET(3.5, "21025", "standing, moderate (lifting 50 lbs., assembling at fast rate)"),
    MET(4.5, "21030", "standing, moderate/heavy work"),
    MET(1.3, "21035", "typing, electric, manual, or computer"),
    MET(2.0, "21040", "walking, less than 2.0 mph, very slow"),
    MET(3.5, "21045", "walking, 3.0 mph, moderate speed, not carrying anything"),
    MET(4.3, "21050", "walking, 3.5 mph, brisk speed, not carrying anything"),
    MET(3.5, "21055", "walking, 2.5 mph slowly and carrying objects less than 25 pounds"),
    MET(4.5, "21060", "walking, 3.0 mph moderately and carrying objects less than 25 pounds, pushing something"),
    MET(4.8, "21065", "walking, 3.5 mph, briskly and carrying objects less than 25 pounds"),
    MET(3.0, "21070", "walk/stand combination, for volunteer purposes"),
    MET(1.5, "21000", "sitting, meeting, general, and/or with talking involved"),
    MET(1.5, "21005", "sitting, light office work, in general"),
    MET(2.5, "21010", "sitting, moderate work"),
    MET(2.3, "21015", "standing, light work (filing, talking, assembling)"),
    MET(2.0, "21016", "sitting, child care, only active periods"),
    MET(3.0, "21017", "standing, child care, only active periods"),
    MET(3.5, "21018", "walk/run play with children, moderate, only active periods"),
    MET(5.8, "21019", "walk/run play with children, vigorous, only active periods"),
    MET(3.0, "21020", "standing, light/moderate work (e.g., pack boxes, assemble/repair, set up chairs/furniture)"),
    MET(3.5, "21025", "standing, moderate (lifting 50 lbs., assembling at fast rate)"),
    MET(4.5, "21030", "standing, moderate/heavy work"),
    MET(1.3, "21035", "typing, electric, manual, or computer"),
    MET(2.0, "21040", "walking, less than 2.0 mph, very slow"),
    MET(3.5, "21045", "walking, 3.0 mph, moderate speed, not carrying anything"),
    MET(4.3, "21050", "walking, 3.5 mph, brisk speed, not carrying anything"),
    MET(3.5, "21055", "walking, 2.5 mph slowly and carrying objects less than 25 pounds"),
    MET(4.5, "21060", "walking, 3.0 mph moderately and carrying objects less than 25 pounds, pushing something"),
    MET(4.8, "21065", "walking, 3.5 mph, briskly and carrying objects less than 25 pounds"),
    MET(3.0, "21070", "walk/stand combination, for volunteer purposes")
]


def karvonen(mets, intensity):
    return intensity * (mets -1) + 1


def from_vo2(vo2):
    return vo2 / 3.5


def stairmaster_mets(setting):
    """
    For use in submaximal tests on the StairMaster 4000 PT step ergometer.

    Howley, Edward T., Dennis L. Colacino, and Thomas C. Swensen. "Factors Affecting the Oxygen Cost of Stepping on an Electronic Stepping Ergometer." Medicine & Science in Sports & Exercise 24.9 (1992): n. pag. NCBI. Web. 10 Nov. 2016.

    args:
        setting (int): the setting of the step ergometer

    Returns:
        float: VO2:subscript:`2max` in kcal/kg*hour
    """
    return 0.556 * 7.45 * setting


def to_kcal(mets, weight):
    """
    Estimates energy expenditure in kilocalories/minute

    Ehrman, Jonathan K. ACSM's Resource Manual for Guidelines for Exercise Testing and Prescription. 6th ed. Philadelphia: Wolters Kluwer Health/Lippincott Williams & Wilkins, 2010. Print.

    args:
        mets (float): MET value
        weight (float): body weight, given in kilograms

    Returns:
        float: kilocalories/minute
    """
    return (mets * 3.5 * weight)/200


def target(vo2max, intensity):
    """
    Used to estimate a MET value for a given VO2max intensity

    Ehrman, Jonathan K. ACSM's Resource Manual for Guidelines for Exercise Testing and Prescription. 6th ed. Philadelphia: Wolters Kluwer Health/Lippincott Williams & Wilkins, 2010. Print.

    args:
        vo2max (float):  A VO2Max value
        intensity (float): The intensity of the activity

    Returns:
        float: MET value
    """
    mets = from_vo2(vo2max)
    targetMets = karvonen(mets, intensity)
    return targetMets
